@@ -18,6 +18,7 @@ from functools import partial
 from itertools import chain
 from typing_extensions import Literal
 import re
+import validators
 
 from scanpy import logging as logg
 from anndata import AnnData
@@ -64,7 +65,8 @@ Arraylike_t = Union[NDArrayA, xr.DataArray]
 InferDims_t = Union[Literal["default", "prefer_channels", "prefer_z"], Sequence[str]]
 Input_t = Union[Pathlike_t, Arraylike_t, "ImageContainer"]
 Interactive = TypeVar("Interactive")  # cannot import because of cyclic dependencies
-
+_ERROR_NOTIMPLEMENTED_LIBID = f"It seems there are multiple `library_id` in `adata.uns[{Key.uns.spatial!r}]`.\n \
+                                Loading multiple images is not implemented (yet), please specify a `library_id`."
 
 __all__ = ["ImageContainer"]
 
@@ -246,7 +248,8 @@ class ImageContainer(FeatureMixin):
         Parameters
         ----------
         img
-            In-memory 2, 3 or 4-dimensional array or a path to an on-disk image.
+            In-memory 2, 3 or 4-dimensional array, a URL to a *Zarr* store (ending in *.zarr*),
+            or a path to an on-disk image.
         %(img_layer)s
         dims
             Where to save channel dimension when reading from a file or loading an array. Valid options are:
@@ -329,14 +332,16 @@ class ImageContainer(FeatureMixin):
     @_load_img.register(Path)
     def _(
         self,
-        img: Pathlike_t,
+        img_path: Pathlike_t,
         chunks: int | None = None,
         dims: InferDimensions | tuple[str, ...] = InferDimensions.DEFAULT,
         **_: Any,
     ) -> xr.DataArray | None:
         def transform_metadata(data: xr.Dataset) -> xr.Dataset:
-            for img in data.values():
-                _assert_dims_present(img.dims)
+            for key, img in data.items():
+                if len(img.dims) != 4:
+                    data[key] = img = img.expand_dims({"z": 1}, axis=-2)  # assume only channel dim is present
+                _assert_dims_present(img.dims, include_z=True)
 
             data.attrs[Key.img.coords] = CropCoords.from_tuple(data.attrs.get(Key.img.coords, _NULL_COORDS.to_tuple()))
             data.attrs[Key.img.padding] = CropPadding.from_tuple(
@@ -347,34 +352,29 @@ class ImageContainer(FeatureMixin):
 
             return data
 
-        img = Path(img)
-        logg.debug(f"Loading data from `{img}`")
+        img_path = str(img_path)
+        is_url, suffix = validators.url(img_path), Path(img_path).suffix.lower()
+        logg.debug(f"Loading data from `{img_path}`")
 
-        if not img.exists():
-            raise OSError(f"Path `{img}` does not exist.")
-
-        suffix = img.suffix.lower()
+        if not is_url and not Path(img_path).exists():
+            raise OSError(f"Path `{img_path}` does not exist.")
 
         if suffix in (".jpg", ".jpeg", ".png", ".tif", ".tiff"):
-            return _lazy_load_image(img, dims=dims, chunks=chunks)
+            return _lazy_load_image(img_path, dims=dims, chunks=chunks)
 
-        if img.is_dir():
+        if suffix == ".zarr" or Path(img_path).is_dir():  # can also be a URL
             if len(self._data):
                 raise ValueError("Loading data from `Zarr` store is disallowed when the container is not empty.")
-
-            self._data = transform_metadata(xr.open_zarr(str(img), chunks=chunks))
-            return
-
-        if suffix in (".nc", ".cdf"):
+            self._data = transform_metadata(xr.open_zarr(img_path, chunks=chunks))
+        elif suffix in (".nc", ".cdf"):
             if len(self._data):
                 raise ValueError("Loading data from `NetCDF` is disallowed when the container is not empty.")
 
-            self._data = transform_metadata(xr.open_dataset(img, chunks=chunks))
-            return
+            self._data = transform_metadata(xr.open_dataset(img_path, chunks=chunks))
+        else:
+            raise ValueError(f"Unable to handle path `{img_path}`.")
 
-        raise ValueError(f"Unknown suffix `{img.suffix}`.")
-
-    @_load_img.register(da.Array)  # type: ignore[no-redef]
+    @_load_img.register(da.Array)
     @_load_img.register(np.ndarray)
     def _(
         self,
@@ -387,7 +387,7 @@ class ImageContainer(FeatureMixin):
 
         return self._load_img(xr.DataArray(img), copy=copy, dims=dims, warn=False)
 
-    @_load_img.register(xr.DataArray)  # type: ignore[no-redef]
+    @_load_img.register(xr.DataArray)
     def _(
         self,
         img: xr.DataArray,
@@ -401,6 +401,8 @@ class ImageContainer(FeatureMixin):
         img = img.copy() if copy else img
         if not ("y" in img.dims and "x" in img.dims and "z" in img.dims):
             _, dims, _, expand_axes = _infer_dimensions(img, infer_dimensions=dims)
+            if TYPE_CHECKING:
+                assert isinstance(dims, Iterable)
             if warn:
                 logg.warning(f"Unable to find `y`, `x` or `z` dimension in `{img.dims}`. Renaming to `{dims}`")
             # `axes` is always of length 0, 1 or 2
@@ -417,7 +419,7 @@ class ImageContainer(FeatureMixin):
         cls,
         adata: AnnData,
         img_key: str | None = None,
-        library_id: str | None = None,
+        library_id: Sequence[str] | str | None = None,
         spatial_key: str = Key.uns.spatial,
         **kwargs: Any,
     ) -> ImageContainer:
@@ -442,6 +444,8 @@ class ImageContainer(FeatureMixin):
         The image container.
         """
         library_id = Key.uns.library_id(adata, spatial_key, library_id)
+        if not isinstance(library_id, str):
+            raise NotImplementedError(_ERROR_NOTIMPLEMENTED_LIBID)
         spatial_data = adata.uns[spatial_key][library_id]
         if img_key is None:
             try:
@@ -577,9 +581,8 @@ class ImageContainer(FeatureMixin):
         **_: Any,
     ) -> xr.Dataset:
         def _rescale(arr: xr.DataArray) -> xr.DataArray:
-            # TODO(michalk8): in skimage==0.19.0, multichannel is deprecated
             scaling_fn = partial(
-                rescale, scale=[scale, scale, 1], preserve_range=True, order=1, multichannel=True, cval=cval
+                rescale, scale=[scale, scale, 1], preserve_range=True, order=1, channel_axis=-1, cval=cval
             )
             dtype = arr.dtype
 
@@ -609,7 +612,7 @@ class ImageContainer(FeatureMixin):
             c = data.x.shape[0] // 2
             # manually reassign coordinates
             library_ids = data.coords["z"]
-            data = data.where((data.x - c) ** 2 + (data.y - c) ** 2 <= c ** 2, other=cval).assign_coords(
+            data = data.where((data.x - c) ** 2 + (data.y - c) ** 2 <= c**2, other=cval).assign_coords(
                 {"z": library_ids}
             )
             data.attrs[Key.img.mask_circle] = True
@@ -714,7 +717,8 @@ class ImageContainer(FeatureMixin):
         self,
         adata: AnnData,
         spatial_key: str = Key.obsm.spatial,
-        library_id: str | None = None,
+        library_id: Sequence[str] | str | None = None,
+        spot_diameter_key: str = "spot_diameter_fullres",
         spot_scale: float = 1.0,
         obs_names: Iterable[Any] | None = None,
         as_array: str | bool = False,
@@ -736,6 +740,9 @@ class ImageContainer(FeatureMixin):
         %(adata)s
         %(spatial_key)s
         %(img_library_id)s
+        spot_diameter_key
+            Key in :attr:`anndata.AnnData.uns` ``['{spatial_key}']['{library_id}']['scalefactors']``
+            where the spot diameter is stored.
         spot_scale
             Scaling factor for the spot diameter. Larger values mean more context.
         obs_names
@@ -770,6 +777,8 @@ class ImageContainer(FeatureMixin):
         if library_id is None:
             try:
                 library_id = Key.uns.library_id(adata, spatial_key=spatial_key, library_id=None)
+                if not isinstance(library_id, str):
+                    raise NotImplementedError(_ERROR_NOTIMPLEMENTED_LIBID)
                 obs_library_ids = [library_id] * adata.n_obs
             except ValueError as e:
                 if "Unable to determine which library id to use" in str(e):
@@ -788,6 +797,8 @@ class ImageContainer(FeatureMixin):
                     f"Trying in `adata.uns[{spatial_key!r}]`"
                 )
                 library_id = Key.uns.library_id(adata, spatial_key=spatial_key, library_id=library_id)
+                if not isinstance(library_id, str):
+                    raise NotImplementedError(_ERROR_NOTIMPLEMENTED_LIBID)
                 obs_library_ids = [library_id] * adata.n_obs
 
         lids = set(obs_library_ids)
@@ -801,7 +812,12 @@ class ImageContainer(FeatureMixin):
 
         for i, (obs, lid) in enumerate(zip(adata.obs_names, obs_library_ids)):
             # get spot diameter of current obs (might be different library ids)
-            diameter = Key.uns.spot_diameter(adata, spatial_key=spatial_key, library_id=lid) * scale
+            diameter = (
+                Key.uns.spot_diameter(
+                    adata, spatial_key=spatial_key, library_id=lid, spot_diameter_key=spot_diameter_key
+                )
+                * scale
+            )
             radius = int(round(diameter // 2 * spot_scale))
 
             # get coords in image pixel space from original space
@@ -1089,7 +1105,7 @@ class ImageContainer(FeatureMixin):
                 - :attr:`anndata.AnnData.uns` ``['{layer_name}_{key_added}']['meshes']`` - list of :class:`numpy.array`,
                   defining a mesh in the spatial coordinates.
 
-            See :mod:`napari`'s `tutorial <https://napari.org/tutorials/fundamentals/shapes.html>`_ for more
+            See :mod:`napari`'s `tutorial <https://napari.org/howtos/layers/shapes.html>`_ for more
             information about different mesh types, such as circles, squares etc.
 
         Returns
@@ -1152,7 +1168,8 @@ class ImageContainer(FeatureMixin):
         kwargs
             Keyword arguments for :func:`dask.array.map_overlap` or :func:`dask.array.map_blocks`, depending whether
             ``depth`` is present in ``fn_kwargs``. Only used when ``chunks != None``.
-            Use ``depth`` to control boundary artifacts if ``func`` requires data from neighboring chunks.
+            Use ``depth`` to control boundary artifacts if ``func`` requires data from neighboring chunks,
+            by default, ``boundary = 'reflect`` is used.
 
         Returns
         -------
@@ -1174,6 +1191,9 @@ class ImageContainer(FeatureMixin):
                 else da.map_blocks(func, arr, **fn_kwargs, **kwargs, dtype=arr.dtype)
             )
 
+        if "depth" in kwargs:
+            kwargs.setdefault("boundary", "reflect")
+
         layer = self._get_layer(layer)
         if new_layer is None:
             new_layer = layer
