@@ -2,18 +2,18 @@
 from __future__ import annotations
 
 from types import MappingProxyType
-from typing import Any, Union, Mapping, Optional, Sequence, TYPE_CHECKING
+from typing import Union  # noqa: F401
+from typing import Any, Mapping, Sequence, TYPE_CHECKING
 from pathlib import Path
 from typing_extensions import Literal
 
 from anndata import AnnData
-from scanpy.plotting._utils import add_colors_for_categorical_sample_annotation
 
 import numpy as np
 import pandas as pd
 
+from matplotlib.axes import Axes
 import seaborn as sns
-import matplotlib.colors as mcolors
 import matplotlib.pyplot as plt
 
 from squidpy._docs import d
@@ -23,25 +23,13 @@ from squidpy.gr._utils import (
     _assert_non_empty_sequence,
 )
 from squidpy.pl._utils import _heatmap, save_fig
+from squidpy.pl._color_utils import Palette_t, _get_palette, _maybe_set_colors
 from squidpy._constants._constants import RipleyStat
 from squidpy._constants._pkg_constants import Key
 
 __all__ = ["centrality_scores", "interaction_matrix", "nhood_enrichment", "ripley", "co_occurrence"]
 
 
-Palette_t = Optional[Union[str, mcolors.ListedColormap]]
-
-
-def _maybe_set_colors(source: AnnData, target: AnnData, key: str, palette: str | None = None) -> None:
-    color_key = Key.uns.colors(key)
-    try:
-        if palette is not None:
-            raise KeyError("Unable to copy the palette when there was other explicitly specified.")
-        target.uns[color_key] = source.uns[color_key]
-    except KeyError:
-        add_colors_for_categorical_sample_annotation(target, key=key, force_update_colors=True, palette=palette)
-
-
 def _get_data(adata: AnnData, cluster_key: str, func_name: str, **kwargs: Any) -> Any:
     key = getattr(Key.uns, func_name)(cluster_key, **kwargs)
     try:
@@ -53,19 +41,6 @@ def _get_data(adata: AnnData, cluster_key: str, func_name: str, **kwargs: Any) -
         ) from None
 
 
-def _get_palette(adata: AnnData, cluster_key: str, categories: Sequence[Any]) -> Mapping[str, Any] | None:
-    try:
-        palette = adata.uns[Key.uns.colors(cluster_key)]
-        if len(palette) < len(categories):
-            raise ValueError(
-                f"Expected to find at least `{len(categories)}` colors, "
-                f"found `{len(palette)}` for key `{cluster_key}`."
-            )
-        return dict(zip(categories, palette))
-    except KeyError:
-        return None
-
-
 @d.dedent
 def centrality_scores(
     adata: AnnData,
@@ -109,7 +84,7 @@ def centrality_scores(
     df[cluster_key] = df.index.values
 
     clusters = adata.obs[cluster_key].cat.categories
-    palette = _get_palette(adata, cluster_key=cluster_key, categories=clusters) if palette is None else palette
+    palette = _get_palette(adata, cluster_key=cluster_key, categories=clusters)
 
     score = scores if score is None else score
     score = _assert_non_empty_sequence(score, name="centrality scores")
@@ -151,6 +126,7 @@ def interaction_matrix(
     figsize: tuple[float, float] | None = None,
     dpi: int | None = None,
     save: str | Path | None = None,
+    ax: Axes | None = None,
     **kwargs: Any,
 ) -> None:
     """
@@ -173,7 +149,7 @@ def interaction_matrix(
     _assert_categorical_obs(adata, key=cluster_key)
     array = _get_data(adata, cluster_key=cluster_key, func_name="interaction_matrix")
 
-    ad = AnnData(X=array, obs={cluster_key: pd.Categorical(adata.obs[cluster_key].cat.categories)})
+    ad = AnnData(X=array, obs={cluster_key: pd.Categorical(adata.obs[cluster_key].cat.categories)}, dtype=array.dtype)
     _maybe_set_colors(source=adata, target=ad, key=cluster_key, palette=palette)
     if title is None:
         title = "Interaction matrix"
@@ -187,6 +163,7 @@ def interaction_matrix(
         figsize=(2 * ad.n_obs // 3, 2 * ad.n_obs // 3) if figsize is None else figsize,
         dpi=dpi,
         cbar_kwargs=cbar_kwargs,
+        ax=ax,
         **kwargs,
     )
 
@@ -208,6 +185,7 @@ def nhood_enrichment(
     figsize: tuple[float, float] | None = None,
     dpi: int | None = None,
     save: str | Path | None = None,
+    ax: Axes | None = None,
     **kwargs: Any,
 ) -> None:
     """
@@ -236,7 +214,7 @@ def nhood_enrichment(
     _assert_categorical_obs(adata, key=cluster_key)
     array = _get_data(adata, cluster_key=cluster_key, func_name="nhood_enrichment")[mode]
 
-    ad = AnnData(X=array, obs={cluster_key: pd.Categorical(adata.obs[cluster_key].cat.categories)})
+    ad = AnnData(X=array, obs={cluster_key: pd.Categorical(adata.obs[cluster_key].cat.categories)}, dtype=array.dtype)
     _maybe_set_colors(source=adata, target=ad, key=cluster_key, palette=palette)
     if title is None:
         title = "Neighborhood enrichment"
@@ -250,6 +228,7 @@ def nhood_enrichment(
         figsize=(2 * ad.n_obs // 3, 2 * ad.n_obs // 3) if figsize is None else figsize,
         dpi=dpi,
         cbar_kwargs=cbar_kwargs,
+        ax=ax,
         **kwargs,
     )
 
@@ -267,6 +246,7 @@ def ripley(
     figsize: tuple[float, float] | None = None,
     dpi: int | None = None,
     save: str | Path | None = None,
+    ax: Axes | None = None,
     legend_kwargs: Mapping[str, Any] = MappingProxyType({}),
     **kwargs: Any,
 ) -> None:
@@ -284,6 +264,8 @@ def ripley(
     plot_sims
         Whether to overlay simulations in the plot.
     %(cat_plotting)s
+    ax
+        Axes, :class:`matplotlib.axes.Axes`.
     legend_kwargs
         Keyword arguments for :func:`matplotlib.pyplot.legend`.
     kwargs
@@ -308,10 +290,19 @@ def ripley(
 
     categories = adata.obs[cluster_key].cat.categories
     palette = _get_palette(adata, cluster_key=cluster_key, categories=categories) if palette is None else palette
-
-    fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
+    if ax is None:
+        fig, ax = plt.subplots(figsize=figsize, dpi=dpi)
+    else:
+        fig = ax.figure
     sns.lineplot(
-        y="stats", x="bins", hue=cluster_key, data=res[f"{mode.s}_stat"], hue_order=categories, palette=palette, ax=ax
+        y="stats",
+        x="bins",
+        hue=cluster_key,
+        data=res[f"{mode.s}_stat"],
+        hue_order=categories,
+        palette=palette,
+        ax=ax,
+        **kwargs,
     )
     if plot_sims:
         sns.lineplot(y="stats", x="bins", ci="sd", alpha=0.01, color="gray", data=res["sims_stat"], ax=ax)
@@ -372,7 +363,7 @@ def co_occurrence(
     clusters = _assert_non_empty_sequence(clusters, name="clusters")
     clusters = sorted(_get_valid_values(clusters, categories))
 
-    palette = _get_palette(adata, cluster_key=cluster_key, categories=categories) if palette is None else palette
+    palette = _get_palette(adata, cluster_key=cluster_key, categories=categories, palette=palette)
 
     fig, axs = plt.subplots(
         1,
