@@ -21,6 +21,7 @@ import os
 from scanpy import logging as logg, settings
 from anndata import AnnData
 
+from dask import array as da, delayed
 from numba import njit, prange
 from scipy.sparse import issparse, spmatrix
 from scipy.cluster import hierarchy as sch
@@ -37,9 +38,13 @@ import numpy as np
 import pandas as pd
 
 from matplotlib import colors as mcolors, pyplot as plt
+from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 import matplotlib as mpl
 
+from skimage import img_as_float32
+from skimage.color import rgb2gray
+
 from squidpy._docs import d
 from squidpy._utils import NDArrayA
 from squidpy.gr._utils import _assert_categorical_obs
@@ -81,9 +86,9 @@ def save_fig(fig: Figure, path: str | Path, make_dir: bool = True, ext: str = "p
 
     if make_dir:
         try:
-            os.makedirs(str(Path.parent), exist_ok=True)
+            path.parent.mkdir(parents=True, exist_ok=True)
         except OSError as e:
-            logg.debug(f"Unable to create directory `{Path.parent}`. Reason: `{e}`")
+            logg.debug(f"Unable to create directory `{path.parent}`. Reason: `{e}`")
 
     logg.debug(f"Saving figure to `{path!r}`")
 
@@ -528,15 +533,20 @@ def _heatmap(
     figsize: tuple[float, float] | None = None,
     dpi: int | None = None,
     cbar_kwargs: Mapping[str, Any] = MappingProxyType({}),
+    ax: Axes | None = None,
     **kwargs: Any,
 ) -> mpl.figure.Figure:
     _assert_categorical_obs(adata, key=key)
 
     cbar_kwargs = dict(cbar_kwargs)
-    fig, ax = plt.subplots(constrained_layout=True, dpi=dpi, figsize=figsize)
+
+    if ax is None:
+        fig, ax = plt.subplots(constrained_layout=True, dpi=dpi, figsize=figsize)
+    else:
+        fig = ax.figure
 
     if method is not None:
-        row_order, col_order, row_link, col_link = _dendrogram(adata.X, method, optimal_ordering=adata.n_obs <= 1500)
+        row_order, col_order, _, col_link = _dendrogram(adata.X, method, optimal_ordering=adata.n_obs <= 1500)
     else:
         row_order = col_order = np.arange(len(adata.uns[Key.uns.colors(key)]))
 
@@ -615,3 +625,31 @@ def _dendrogram(data: NDArrayA, method: str, **kwargs: Any) -> tuple[list[int],
     col_order = col_dendro["leaves"]
 
     return row_order, col_order, row_link, col_link
+
+
+def sanitize_anndata(adata: AnnData) -> None:
+    """Transform string annotations to categoricals."""
+    adata._sanitize()
+
+
+def _assert_value_in_obs(adata: AnnData, key: str, val: Sequence[Any] | Any) -> None:
+    if key not in adata.obs:
+        raise KeyError(f"Key `{key}` not found in `adata.obs`.")
+    if not isinstance(val, list):
+        val = [val]
+    val = set(val) - set(adata.obs[key].unique())
+    if len(val) != 0:
+        raise ValueError(f"Values `{val}` not found in `adata.obs[{key}]`.")
+
+
+def _to_grayscale(img: NDArrayA | da.Array) -> NDArrayA | da.Array:
+    if img.shape[-1] != 3:
+        raise ValueError(f"Expected channel dimension to be `3`, found `{img.shape[-1]}`.")
+
+    if isinstance(img, da.Array):
+        img = da.from_delayed(delayed(img_as_float32)(img), shape=img.shape, dtype=np.float32)
+        coeffs = np.array([0.2125, 0.7154, 0.0721], dtype=img.dtype)
+
+        return img @ coeffs
+
+    return rgb2gray(img)
