@@ -8,6 +8,8 @@ import scanpy as sc
 import numpy as np
 import pandas as pd
 
+import matplotlib.pyplot as plt
+
 from squidpy import gr, pl
 from tests.conftest import DPI, PlotTester, PlotTesterMeta
 
@@ -57,6 +59,13 @@ class TestGraph(PlotTester, metaclass=PlotTesterMeta):
 
         pl.nhood_enrichment(adata, cluster_key=C_KEY)
 
+    def test_plot_nhood_enrichment_ax(self, adata: AnnData):
+        gr.spatial_neighbors(adata)
+        gr.nhood_enrichment(adata, cluster_key=C_KEY)
+
+        fig, ax = plt.subplots(figsize=(2, 2), constrained_layout=True)
+        pl.nhood_enrichment(adata, cluster_key=C_KEY, ax=ax)
+
     def test_plot_nhood_enrichment_dendro(self, adata: AnnData):
         gr.spatial_neighbors(adata)
         gr.nhood_enrichment(adata, cluster_key=C_KEY)
